@@ -2,7 +2,7 @@ package Plack::Request;
 use strict;
 use warnings;
 use 5.008_001;
-our $VERSION = '1.0040';
+our $VERSION = '1.0052';
 
 use HTTP::Headers::Fast;
 use Carp ();
@@ -12,6 +12,7 @@ use Plack::Request::Upload;
 use Stream::Buffered;
 use URI;
 use URI::Escape ();
+use Cookie::Baker ();
 
 use HTTP::Entity::Parser;
 use WWW::Form::UrlEncoded qw/parse_urlencoded_arrayref/;
@@ -61,20 +62,7 @@ sub cookies {
     }
 
     $self->env->{'plack.cookie.string'} = $self->env->{HTTP_COOKIE};
-
-    my %results;
-    my @pairs = grep m/=/, split "[;,] ?", $self->env->{'plack.cookie.string'};
-    for my $pair ( @pairs ) {
-        # trim leading trailing whitespace
-        $pair =~ s/^\s+//; $pair =~ s/\s+$//;
-
-        my ($key, $value) = map URI::Escape::uri_unescape($_), split( "=", $pair, 2 );
-
-        # Take the first one like CGI.pm or rack do
-        $results{$key} = $value unless exists $results{$key};
-    }
-
-    $self->env->{'plack.cookie.parsed'} = \%results;
+    $self->env->{'plack.cookie.parsed'} = Cookie::Baker::crush_cookie($self->env->{'plack.cookie.string'});
 }
 
 sub content {
@@ -105,9 +93,9 @@ sub headers {
         $self->{headers} = HTTP::Headers::Fast->new(
             map {
                 (my $field = $_) =~ s/^HTTPS?_//;
-                ( $field => $env->{$_} );
+                ( lc($field) => $env->{$_} );
             }
-                grep { /^(?:HTTP|CONTENT)/i } keys %$env
+                grep { /^(?:HTTP|CONTENT)_/ } keys %$env
             );
     }
     $self->{headers};
@@ -263,11 +251,6 @@ sub _buffer_length_for {
 
 sub _parse_request_body {
     my $self = shift;
-    if ( !$self->env->{CONTENT_TYPE} ) {
-        $self->env->{'plack.request.body_parameters'} = [];
-        $self->env->{'plack.request.upload'} = Hash::MultiValue->new();
-        return;
-    }
 
     my ($params,$uploads) = $self->request_body_parser->parse($self->env);
     $self->env->{'plack.request.body_parameters'} = $params;
@@ -438,7 +421,7 @@ strings that are sent by clients and are URI decoded.
 If there are multiple cookies with the same name in the request, this
 method will ignore the duplicates and return only the first value. If
 that causes issues for you, you may have to use modules like
-CGI::Simple::Cookie to parse C<< $request->header('Cookies') >> by
+CGI::Simple::Cookie to parse C<< $request->header('Cookie') >> by
 yourself.
 
 =item query_parameters
@@ -497,11 +480,20 @@ Shortcut to $req->headers->content_encoding.
 
 =item content_length
 
-Shortcut to $req->headers->content_length.
+Returns the raw value of the Content-Length header.
+
+Before version 0.9925, this method was a shortcut for
+C<< $req->headers->content_length >>.
 
 =item content_type
 
-Shortcut to $req->headers->content_type.
+Returns the raw value of the Content-Type header.
+
+If you want just the MIME type, without any attributes like charset, use
+C<< $req->headers->content_type >>.  See also L<HTTP::Headers/content_type>.
+
+Before version 0.9925, this method was a shortcut for
+C<< $req->headers->content_type >>.
 
 =item header
 
